#include <ESP8266WiFi.h>
#include <MQTT.h>
#include <Servo.h>
 
// Wi-Fi credentials
const char ssid[] = "StudentResidences";
const char pass[] = "";  // Update with the correct Wi-Fi password
 
// Create instances
WiFiClient net;
MQTTClient client;
Servo myservo;
 
void connectWiFi() {
  Serial.print("Connecting to Wi-Fi...");
  WiFi.begin(ssid, pass);
 
  // Wait until connected
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(1000);
  }
  Serial.println("\nConnected to Wi-Fi!");
}
 
void connectMQTT() {
  String clientId = "Hamishservo_" + String(WiFi.macAddress());  // Unique client ID using MAC address
  Serial.print("Connecting to MQTT broker with Client ID: ");
  Serial.println(clientId);
 
  unsigned long startMillis = millis();
  while (!client.connect(clientId.c_str(), "", "")) {
    if (millis() - startMillis > 10000) {  // Timeout after 10 seconds
      Serial.println("\nFailed to connect to MQTT Broker!");
      return;  // Exit the function if it fails to connect
    }
    Serial.print(".");
    delay(1000);
  }
 
  Serial.println("\nConnected to MQTT Broker!");
  client.subscribe("HamishDB");  // Subscribe to the topic
}
 
void messageReceived(String &topic, String &payload) {
  // Handle incoming messages (simple number payload)
  Serial.println("Incoming message: " + topic + " - " + payload);

  // Convert the payload directly to a float
  float dustDensity = payload.toFloat();  // Convert the incoming string to a float

  // Print the dust density value
  Serial.println("Received dust density: " + String(dustDensity));

  // Adjust the servo position based on the dust density value
  if (dustDensity < 50) {
    myservo.write(0);
  } else if (dustDensity >= 50 && dustDensity < 150) {
    myservo.write(90);
  } else if (dustDensity >= 150) {
    myservo.write(180);
  }
}


void setup() {
  Serial.begin(115200);
 
  // Connect to Wi-Fi
  connectWiFi();
 
  // Initialize MQTT client and set up message handler
  client.begin("134.36.5.199", 1883, net);
  client.onMessage(messageReceived);
 
  // Connect to MQTT broker
  connectMQTT();
 
  // Attach servo to pin D4
  myservo.attach(D4, 500, 2400);  // Servo range (500 to 2400 microseconds)
}
 
void loop() {
  // Keep the MQTT client alive
  client.loop();
 
  // Reconnect to MQTT if disconnected
  if (!client.connected()) {
    Serial.println("MQTT client disconnected. Reconnecting...");
    connectMQTT();
  }
 
  delay(10);  // Small delay to avoid blocking
}